<?php
/**
 * Connection statistics database functions
 *
 * Create database and tables
 * Log connections, resources access
 *
 *
 *
 * TABLES:
 *
 * account: list user accounts
 * 	- id(key): account unique id found in .usr file
 * 	- name: account friendly name
 * 	- admin (text, 0/1): 1 if user account is administrator
 * 	- creationDate: timestamp of account creation
 *
 * resource: list of created resources
 * 	- id(key): resource unique id found in .res file
 * 	- name: resource friendly name
 * 	- creationDate: timestamp of account creation
 *
 * connection: log of user connections
 *	- id (key): sequential index
 * 	- ip: user IP (converted to int)
 * 	- accountId: account id (account table key)
 * 	- ts: timestamp of connection
 * 	- duration: number of seconds of connection
 * 	- pwdErr: number of password typed errors
 * 	- hostname: resolved (from ip) user's hostname
 * 	- pseudo: user typed pseudo (if applicable)
 *
 * Note: connection id is put in dbId general var (cfGSetVar)
 *
 *
 * resAccess
 * 	- connectionId: id of connection (connection table key)
 * 	- resourceId: id of resource (resource table key)
 * 	- pages: number of pages viewed
 *
 * contentAccess
 * 	- id: (table key)
 * 	- connectionId: id of connection (connection table key)
 * 	- resourceId: id of resource (resource table key)
 * 	- pages: number of pages viewed
 *
 *
 *
 * PHP version 5
 *
 * LICENSE: This source file is subject to version 3.0 of the PHP license
 * that is available through the world-wide-web at the following URI:
 * http://www.php.net/license/3_0.txt.  If you did not receive a copy of
 * the PHP License and are unable to obtain it through the web, please
 * send a note to license@php.net so we can mail you a copy immediately.
 *
 * @category   NA
 * @package    NA
 * @author     Nicolas Bruley / Peer 2 World <contact@weezo.net>
 * @copyright  2005-2009 Nicolas Bruley / Peer 2 World
 * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
 * @version    CVS: $Id:$
 * @link       http://www.weezo.net
 * @since      File available since Release 1.0.0
 */


if(class_exists('PDO')) {
	define('W_DATABASE_ENGINE','PDO_SQLITE3');
	define('W_STATS_DATABASE_FILENAME',cfAppDataDir().'/cnxStats3.db');
	define('W_FILES_DATABASE_FILENAME',cfAppDataDir().'/filesCache3.db');
}
elseif(class_exists('SQlite3')){
	define('W_DATABASE_ENGINE','SQLITE3');
	define('W_STATS_DATABASE_FILENAME',cfAppDataDir().'/cnxStats3.db');
	define('W_FILES_DATABASE_FILENAME',cfAppDataDir().'/filesCache3.db');
}
else {
	define('W_DATABASE_ENGINE','SQLITE2');
	define('W_STATS_DATABASE_FILENAME',cfAppDataDir().'/cnxStats2.db');
	define('W_FILES_DATABASE_FILENAME',cfAppDataDir().'/filesCache.db');
}


/**
 * @Dump 100 first lines of each tables of a given database
 *
 * @param string $completeFilename, let to false to use global va $db;
 * @param integer $limit: max nb of rows per table
 */
function dbDumpDatabase($completeFilename=false,$limit=100){
	global $db;

	if(!$db && !$completeFilename) $completeFilename=W_STATS_DATABASE_FILENAME;

	if($completeFilename){
		if(!file_exists($completeFilename)) {
			throw new Exception('dbDumpDatabase: File not found');
			return;
		}
		$db=dbOpenDB($completeFilename);
	}
	if(!$db) {
		throw new Exception('dbDumpDatabase: error opening database');
		return;
	}

	foreach (dbArrayQuery('SELECT * FROM sqlite_master',$db) as $t) if($t['type']=='table'){
		dbDisplayTable($t['name'],array('limit'=>$limit));
	}
	dbClose($db);
}

/**
 * @desc Create database
 *
 */
function dbCreateStatsDB(){
	global $db;

	// Open DB
	if(!($db=dbOpenDB())) return false;

	// Create and populate tables if needed
	// Database version
	if(!dbTableExists('databaseVersion',$db)) {
		dbExec('CREATE TABLE databaseVersion (version INTEGER)',$db);
		dbExec('INSERT INTO databaseVersion VALUES ("5")',$db);
	}

	// account
	if(!dbTableExists('account',$db)) {
		dbExec('CREATE TABLE account (id PRIMARY KEY, name TEXT, admin TEXT, creationDate TIMESTAMP)',$db);
	}
	dbPopulateAccount($db);

	// resource
	if(!dbTableExists('resource',$db)) {
		dbExec('CREATE TABLE resource (id PRIMARY KEY, name TEXT, creationDate TIMESTAMP)',$db);
		dbPopulateResource($db);
	}

	// connection (user connections)
	if(!dbTableExists('connection',$db)) dbExec('CREATE TABLE connection (id INTEGER PRIMARY KEY, ip INTEGER, accountId TEXT , ts TIMESTAMP, duration INTEGER, pwdErr INTEGER, hostname TEXT, pseudo TEXT, browser TEXT)',$db);

	// resAccess (resources access)
	if(!dbTableExists('resAccess',$db)) dbExec('CREATE TABLE resAccess (connectionId INTEGER, resourceId INTEGER, pages INTEGER, UNIQUE(connectionId, resourceId))',$db);

	// resContent (viewed content)
	if(!dbTableExists('contentAccess',$db)) dbExec('CREATE TABLE contentAccess (id INTEGER PRIMARY KEY, connectionId INTEGER, resourceId INTEGER, content TEXT, ts TIMESTAMP, UNIQUE(connectionId, resourceId, content))',$db);
	else{
		// Clean "expired" viewed content
		if(cfGGetVar('historyLogViewedContent') && (int)cfGGetVar('historyLogViewedContentDays')){

			dbExec('DELETE FROM contentAccess WHERE ts<'.(time()-3600*24*(int)cfGGetVar('historyLogViewedContentDays')),$db);
		}
	}

	// Close DB
	dbClose($db); $db=false;
	return true;
}

/**
 * @desc Destroy and recreate database
 *
 * @return boolean: true on success
 */
function dbResetDB(){
	if(!@unlink(W_STATS_DATABASE_FILENAME)) return false;
	dbCreateStatsDB();
	return true;
}

/**
 * @dest generate an HTML table from a table, a query or an array
 *
 * @param mixed $table : database table name, sqlite request or array
 * @param array $options : 'limit'=>max number of rows
 * @param database $database: if set, overrides global $db parameter
 */
function dbDisplayTable($table,$options=array(),$database=false){
	global $db;
	if($database) $db=$database;
	else if(!$db) $db=dbOpenDB();

	$limit=(isset($options['limit']))?' LIMIT '.$options['limit']:'';

	if(is_array($table)){
		$result=$table;
	}
	elseif (dbTableExists($table,$db)){
		$query='SELECT * FROM '.$table.$limit;
		$result=@dbArrayQuery($query, $db);
		echo "<b>".$table.'</b><br>';
	}
	else{
		$result=@dbArrayQuery($table, $db);
	}
	if(!$result || count($result)==0) echo 'Table vide<br/><br/>';
	else{
		echo '<table style="border:1px solid black; color:#550; font-family:arial; font-size:10px" cellspacing=0 cellpadding=0><tr>';
		foreach ($result[0] as  $key=>$value) echo '<td style="font-weight:bold; padding:3px; background:#550; color:white; font-weight:bold; font-family:arial; font-size:10px">'.$key.'</td>';
		echo '</tr>';
		foreach ($result as $key=>$value){
			echo '<tr>';
			foreach ($value as $k2=>$v2) {
				echo '<td style="border:1px dashed #AAA; padding:3px">';
				 if($k2==='ts') echo date('d/m/y h:i:s',$v2); else echo $v2;
				echo '</td>';
			}
			echo "</tr>\n";
		}
		echo "</table>\n";
	}
}

/**
 * @desc fills account table from user files
 *
 */
function dbPopulateAccount(&$db){
	// Build an id=>name array of in-db users
	$c=array();
	foreach (dbArrayQuery('SELECT * FROM account',$db) as $value) $c[$value['id']]=$value['name'];

	dbBeginTransaction($db);
	foreach (WEnv::userConfigs() as $id=>$uc){
		if(!isset($c[$id])) dbExec("INSERT INTO account VALUES ('" . $id . "' ,'" .dbEscapeString($uc->getVar('name'),$db) . "' ,'" .(int)$uc->isAdministrator(). "', '".time()."')",$db);
		elseif ($c[$id]!==$uc->getVar('name'))  dbExec("UPDATE account SET name='".dbEscapeString($uc->getVar('name'),$db)."' WHERE id='".$id."'",$db);
	}
	dbCommitTransaction($db);
}

/**
 * @desc fills resource table from resource files
 *
 */
function dbPopulateResource(&$db){
	$files = glob(cfAppDataDir().'/res/*.res');

	dbBeginTransaction($db);

	foreach ($files as $filename) {
		$acc=cfParse_ini_file($filename);

		if(isset($acc['id']) && isset($acc['name']) && $acc['id']!="")
			dbExec("INSERT INTO resource VALUES ('" . $acc['id'] . "' ,'" .dbEscapeString($acc['name'],$db). "','".filectime($filename)."')",$db);
	}
	dbCommitTransaction($db);
}

/**
 * @desc log new user connection
 * @param string host : visitor's hostname
 *
 */
function dbLogNewConnection($host=''){
	// No Stats => exit
	if(!cfGGetVar('logUserActivity')) return;

	global $db;
	// Open DB
	$db=dbOpenDB(); if($db==false) return false;

	// Set subUser name / pseudo
	if(WEnv::user()->getVar('accountType')=='list') $userName=WEnv::user()->subUser()->name();
	else $userName=cfUGetVar('pseudo');

	// INSERT
	dbExec("INSERT INTO connection VALUES (NULL, '" . $_SERVER['REMOTE_ADDR'] . "' ,'" . dbEscapeString(cfUGetVar('id'),$db) . "' ,'" .time() . "','0','0','".$host."','".dbEscapeString($userName,$db)."','".dbEscapeString(cfBGetVar('name'),$db)."')",$db);

	// Get id of inserted record
	cfGSetVar('dbId',dbLastInsertId($db));

	// Close DB
	dbClose($db);
}

/**
 * @desc log resource access
 *
 */
function dbLogResourceAccess(){
	// No Stats => exit
	if(!cfGGetVar('logUserActivity')) return;
	// No login for async requests
	$logPagesAccess=(isset($_POST['noSt']) || isset($_GET['noSt']) || isset($_POST['asyncRequest']) || isset($_GET['asyncRequest']));

	// If 1st access has already been logged, and no page access is needed, exit
	if(cfRGetVar('logUserActivityAccessLogged') && !$logPagesAccess) return;

	// Open DB
	global $db;
	$db=dbOpenDB(); if($db==false) return false;

	// Log 1st resource access
	if(!cfRGetVar('logUserActivityAccessLogged')){
		dbExec("INSERT INTO resAccess VALUES ('".cfGGetVar('dbId')."', '" .cfRGetVar('id')."' ,'1')",$db);
		cfRSetVar('logUserActivityAccessLogged',true);
	}

	// Log number of access to this resource
	if($logPagesAccess){
		$nb=(dbSingleQuery("SELECT pages FROM resAccess WHERE connectionId='".cfGGetVar('dbId')."' AND resourceId='" .cfRGetVar('id')."'",$db));
		dbExec("UPDATE resAccess SET pages='".($nb+1)."' WHERE connectionId='".cfGGetVar('dbId')."' AND resourceId='" .cfRGetVar('id')."'",$db);
	}

	// Close DB
	dbClose($db);
}

/**
 * @desc log content access
 *
 */
function dbLogContentAccess($completeFilename){
	// No Stats => exit
	if(!cfGGetVar('logUserActivity') || !cfGGetVar('historyLogViewedContent')) return;

	// Open DB
	global $db;
	$db=dbOpenDB(); if($db==false) return false;

	// Log content access
	if(!dbExec("INSERT INTO contentAccess VALUES (NULL, '".cfGGetVar('dbId')."', '" .cfRGetVar('id')."' ,'" .dbEscapeString($completeFilename,$db)."',".time().")",$db)){
		dbExec("UPDATE contentAccess SET ts=".time()." WHERE connectionId='".cfGGetVar('dbId')."' AND resourceId='".cfRGetVar('id')."' AND content='" .dbEscapeString($completeFilename,$db)."'",$db);
	}

	// Close DB
	dbClose($db);
}



/**
 * @desc log user disconnection
 * @param $sessId : optional (application call only) : session Id
 *
 */
function dbLogDisconnection($dbId=false){
	if(!cfGGetVar('logUserActivity')) return;

	// Open DB
	$db=dbOpenDB(); if($db==false) return false;

	// Retreive row id from saved session var if not passed as a parameter
	if($dbId===false) $dbId=cfGGetVar('dbId');

	// Set duration
	$ts=@dbSingleQuery("SELECT ts FROM connection WHERE id='".$dbId."'",$db);
	dbExec("UPDATE connection SET duration='".(time()-$ts)."' WHERE id='".$dbId."'",$db);

	// Close DB
	dbClose($db);
}

/**
 * @desc Update information about a given user group
 *
 * @param string $userId: group id
 * @param string $userName: group name
 * @param boolean $admin: true if this group is admin
 * @return unknown
 */
function dbUpdateAccount($userId, $userName, $admin){
	global $db;
	if(!$db) {$db=dbOpenDB(); if($db==false) return false;}

	$query="INSERT INTO account VALUES ('" & id & "' ,'" & name & "','" & admin & "','".time()."')";
	dbExec($query, $db);
	$query="UPDATE account SET name='" & name & "' WHERE id='" & id & "'";
	dbExec($query, $db);
	dbClose($db);
}



/********************************************************************************************************************
 * DATABASE Wrappers
 ********************************************************************************************************************/


/**
 * @desc open stats database and put database resource in $db
 *
 * @return boolean : true if db opened, else false
 */
function dbOpenDB($dbFilename=false){
	$file=($dbFilename)?$dbFilename:W_STATS_DATABASE_FILENAME;

	if(W_DATABASE_ENGINE=='SQLITE2') return @sqlite_open($file);
	else if(W_DATABASE_ENGINE=='SQLITE3'){
		try{$db=new SQLite3($file);} catch(Exception $e){return false;}
		return $db;
	}
	else if(W_DATABASE_ENGINE=='PDO_SQLITE3'){
 		try{$db=new PDO('sqlite:'.$file);} catch(PDOException $e){
 			//cfVarDump($e->getMessage());
 			return false;
 		}
		return $db;
	}

	return false;
}

/**
 * @desc close database
 *
 * @param unknown_type $db
 */
function dbClose(&$db){
	if(!isset($db) && !$db) return false;

	if(W_DATABASE_ENGINE=='SQLITE2') return @sqlite_close($db);
	else if(W_DATABASE_ENGINE=='SQLITE3') return @$db->close();
	else if(W_DATABASE_ENGINE=='PDO_SQLITE3') {
		$db=null;
	}
}

/**
 * @desc Exec a query and just return OK/NOK result
 *
 * @param string $query: SQL query
 * @param objec $db: database
 * @return bool
 */
function dbExec($query, &$db){
	// Transaction
	if((W_DATABASE_ENGINE=='SQLITE2' || W_DATABASE_ENGINE=='SQLITE3') && isset($_ENV['dbTransactionBuffer'])){
		if(substr($query,-1)!=';') $query.=';';
		$_ENV['dbTransactionBuffer'].=$query;
		return true;
	}

	if(W_DATABASE_ENGINE=='SQLITE2') return @sqlite_query($query,$db);
	elseif(W_DATABASE_ENGINE=='SQLITE3') return @$db->exec($query);
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') {
		//cfVarDump($query);
		$res=$db->query($query);
		//cfVarDump($res->errorInfo());
		//cfVarDump (!($res===false));
		return !($res===false);

		//return !(@$db->exec($query)===false);
	}
	return false;
}


/**
 * @desc Run a request and return result as an associative array
 *
 * @param string $query: SQL query
 * @param object $db
 * @return array
 */
function dbArrayQuery($query, &$db){
	if(W_DATABASE_ENGINE=='SQLITE2') return @sqlite_array_query($query, $db, SQLITE_ASSOC);
	elseif(W_DATABASE_ENGINE=='SQLITE3') {
		 $results=@$db->query($query);
		 $rows=array();
		 if(!$results) return $rows;
		 while ($row = $results->fetchArray()) $rows[]=$row;
		 return $rows;
	}
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') {
		 $results=$db->query($query);
		 if(!$results) return false;

		 return $results->fetchAll(PDO::FETCH_ASSOC);
	}
}

/**
 * @desc Run a request and return first col of first row
 *
 * @param string $query: SQL query
 * @param object $db
 * @return string
 */
function dbSingleQuery($query,&$db){
	if(W_DATABASE_ENGINE=='SQLITE2') return @sqlite_single_query($query, $db);
	elseif(W_DATABASE_ENGINE=='SQLITE3') return $db->querySingle($query);
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') {
		 $results=$db->query($query);
		 if(!$results) return false;
		 $row=$results->fetch(PDO::FETCH_NUM);
		 return $row[0];
	}
}

/**
 * @desc Begin DB transaction. Note: only work with dbExec
 *
 * @param mixed $db
 */
function dbBeginTransaction($db){
	if(W_DATABASE_ENGINE=='SQLITE2') dbExec('BEGIN TRANSACTION');
	elseif(W_DATABASE_ENGINE=='SQLITE3') dbExec('BEGIN TRANSACTION');
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') $db->beginTransaction();
}

/**
 * @desc Commit DB transaction
 *
 * @param mixed $db
 * @return boolean
 */
function dbCommitTransaction($db){
	if(W_DATABASE_ENGINE=='SQLITE2') dbExec('COMMIT');
	elseif(W_DATABASE_ENGINE=='SQLITE3') dbExec('COMMIT');
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') $result=@$db->commit();
	return $result;
}


/**
 * @desc check wether a table exists or not in $db
 *
 * @param sqlite resource $sqlite : database
 * @param string $table : name of table
 * @return boolean : result
 */
function dbTableExists($table,&$db){
	return count(dbArrayQuery("SELECT name FROM sqlite_master WHERE type='table' AND name='$table'",$db))>0;
}


/**
 * @desc sqlite_escape_string wrapper
 *
 * @param string $string: string to be escaped
 * @return srting
 */
function dbEscapeString($string,&$db){
	static $tmpDB;

	if(W_DATABASE_ENGINE=='SQLITE2') return sqlite_escape_string($string);
	elseif(W_DATABASE_ENGINE=='SQLITE3') return SQlite3::escapeString($string);
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') {
		// PDO requires a database to escape string. If database not available, create a temp database in memory
		if(!$db){
			if(!isset($tmpDB)) $tmpDB=new PDO('sqlite::memory:');
	 		$q=substr($tmpDB->quote($string),1); return substr($q,0,strlen($q)-1);
		}
		$q=substr($db->quote($string),1); return substr($q,0,strlen($q)-1);
	}
}

/**
 * Returns the row ID of the most recent INSERT into the database
 *
 * @param mixed $db
 * @return integer
 */
function dbLastInsertId(&$db){
	if(W_DATABASE_ENGINE=='SQLITE2') return sqlite_last_insert_rowid($db);
	elseif(W_DATABASE_ENGINE=='SQLITE3') return SQlite3::lastInsertRowID();
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') return $db->lastInsertId();
}

function dbLastError(&$db){
	if(W_DATABASE_ENGINE=='SQLITE2') return sqlite_last_insert_rowid($db);
	elseif(W_DATABASE_ENGINE=='SQLITE3') return SQlite3::lastInsertRowID();
	elseif(W_DATABASE_ENGINE=='PDO_SQLITE3') return $db->errorInfo();
}
?>