<?php

class FolderSync{
	private $sourceFolder;
	private $destFolder;
	private $controlMethod='md5';
	private $ok=false;

	private $outputType='html';
	private $lineBreak="<br>\n";

	private $debug=false;

	private $processed=0;
	private $updated=0;
	private $skipped=0;
	private $removed=0;
	private $created=0;
	private $errors=0;

	private $excludedSourceFolders=array();
	private $excludedFileTypes=array();

	private $outputFunction=false;

	/**
	 * @desc Constructor
	 *
	 * @param string $sourceFolder: source folder
	 * @param string $destFolder: destination folder
	 */
	function FolderSync($sourceFolder,$destFolder){
		$this->sourceFolder=$sourceFolder;
		$this->destFolder=$destFolder;
		if(!is_dir($sourceFolder)) throw new Exception('FolderSync: source folder is not an existing folder',8);
		elseif(!is_dir($destFolder)) throw new Exception('FolderSync: destination folder is not an existing folder',8);
		else $ok=true;
	}

	/**
	 * @desc Update source folder for new sync
	 *
	 * @param string $sourceFolder
	 */
	function setSourceDir($sourceFolder){
		$this->sourceFolder=$sourceFolder;
		if(!is_dir($sourceFolder)) throw new Exception('FolderSync: source folder is not an existing folder',8);
	}

	/**
	 * @desc Update destination folder for new sync
	 *
	 * @param unknown_type $destFolder
	 */
	function setDestDir($destFolder){
		$this->destFolder=$destFolder;
		if(!is_dir($destFolder)) throw new Exception('FolderSync: destination folder is not an existing folder',8);
	}

	/**
	 * @desc Set a list of excluded source folders
	 *
	 * @param array $sourceFolders
	 */
	function setExcludedSourceFolders($sourceFolders){$this->excludedSourceFolders=$sourceFolders;}

	/**
	 * @desc Set a list of excluded file extensions
	 *
	 * @param array $extensions: array of extensions (dot not included)
	 */
	function setExcludedFileTypes($extensions){$this->excludedFileTypes=$extensions;}

	/**
	 * @desc define a function for displaying creation/update/deletion messages
	 *
	 * @param string $functionName
	 */
	function setOutputFunction($functionName){$this->outputFunction=$functionName;}

	/**
	 * @desc Set update criteria: md5, filemtime, file size or always replace
	 *
	 * @param string $method: md5, mtime, no, filesize
	 */
	function setControlMethod($method){
		if(in_array($method,array('md5','no','mtime','filesize'))) $this->controlMethod=$method;
		else throw new Exception('FolderSync: invalid control method name, must be md5, mtime or no',8);
	}

	/**
	 * @desc Set debug mode
	 *
	 * @param boolean $activated
	 */
	function setDebug($activated=true){$this->debug=$activated;}

	/**
	 * @desc Return tag encapsulating message
	 *
	 * @param string $action
	 * @return string
	 */
	function tag($action=false){
		if($this->outputType!='html') return '';
		switch ($action){
			case 'create': return '<span style="color:green">';
			case 'delete': return '<span style="color:red">';
			case false: return '</span>';
			default: return '<span>';
		}
	}

	/**
	 * @desc Start synchronization
	 *
	 */
	function go(){
		$this->cleanDestination($this->destFolder);

		if($this->debug) $this->sendAction('----------','-----------------------','');

		$this->updateDestination($this->sourceFolder);
	}

	/**
	 * @desc Return true if file needs update
	 *
	 * @param string $sourceFile
	 * @param string $destFile
	 * @return boolean
	 */
	function needUpdate($sourceFile, $destFile){
		switch ($this->controlMethod){
			case 'no': return true;
			case 'md5':
				if(filesize($sourceFile)!=filesize($destFile)) return true;
				return md5_file($sourceFile)!=md5_file($destFile);
			case 'mtime': return filemtime($sourceFile)!=filemtime($destFile);
			case 'filesize': return filesize($sourceFile)!=filesize($destFile);
			case 'filesizeormtime': return filemtime($sourceFile)!=filemtime($destFile)||filesize($sourceFile)!=filesize($destFile);
		}
		return false;
	}

	/**
	 * @desc Remove uneeded files from destination folder and update existing files
	 *
	 * @param string $destFolder: destination folder to process
	 */
	function cleanDestination($destFolder){
		$files=array();
		$folders=array();
		$sourceFolder=$this->sourceFolder.substr($destFolder,strlen($this->destFolder));

		//if($this->debug) $this->sendAction($sourceFolder.' / '.$destFolder,' clean process');

		if($dh = opendir($destFolder)){
			// Browse for files & folders
	        while (($file = readdir($dh)) !== false) {
	        	if($file=='.'||$file=='..') continue;
	        	if(is_file($destFolder.'/'.$file)) $files[]=$file; else $folders[]=$file;
	        }
	        closedir($dh);

			/**
			 * Process files
			 */
			foreach ($files as $file){
				$destFile=$destFolder.'/'.$file;
				$sourceFile=$sourceFolder.'/'.$file;

				$this->processed++;

				// Remove file
				if(!is_file($sourceFile)){
					$this->sendAction($sourceFile,$destFile,'delete');
					if($this->debug || @unlink($destFile)) $this->removed++;
					elseif(!$this->debug) $this->sendError($destFile,'fileDelete');
				}

				// Check for update
				elseif ($this->needUpdate($sourceFile,$destFile)){
					$this->sendAction($sourceFile,$destFile,'update');

					// Copy source on destination
					if($this->debug || @copy($sourceFile,$destFile)) $this->updated++;
					else $this->sendError($destFile,'fileUpdate');
				}
				// No update needed
				else $this->sendAction($sourceFile,$destFile,'skip');
			}

			/**
			 * Process folders
			 */
			foreach ($folders as $dir){
				$destFile=$destFolder.'/'.$dir;
				$sourceFile=$sourceFolder.'/'.$dir;

				// Remove folder if source doesn't exist
				if(!is_dir($sourceFile)){
					if($this->debug || @cfUnlinkDir($destFile)){
						$this->removed++;
					}
					elseif (!$this->debug) $this->sendError($destFile,'folderDelete');
				}

				// Recurse process folder
				else $this->cleanDestination($destFile);
			}
		}
        else{
        	$this->sendError($destFolder,'folderOpen');
        }
	}

	/**
	 * @desc copy inexisting destination files
	 *
	 * @param string $sourceFolder
	 */
	function updateDestination($sourceFolder){
		$files=array();
		$folders=array();
		$destFolder=$this->destFolder.substr($sourceFolder,strlen($this->sourceFolder));

		//if($this->debug) $this->sendAction($sourceFolder.' / '.$destFolder,' update process');

		// Open source folder for reading
		if(!($dh=opendir($sourceFolder))){
			$this->sendError($sourceFolder,'folderOpen');
			return ;
		}
		// Browse files and dirs
		while (($file = readdir($dh)) !== false){
			if($file=='.'||$file=='..') continue;
			if(is_dir($sourceFolder.'/'.$file)) $folders[]=$file; else $files[]=$file;
		}
		closedir($dh);

		/**
		 * Process files
		 */
		foreach ($files as $file){
			$destFile=$destFolder.'/'.$file;
			$sourceFile=$sourceFolder.'/'.$file;

			// Copy file
			if(!file_exists($destFile) && !in_array(cfFileExtension($sourceFile),$this->excludedFileTypes)){
				$this->processed++;
				$this->sendAction($sourceFile,$destFile,'create');
				if($this->debug || @copy($sourceFile,$destFile)) $this->created++;
				elseif(!$this->debug) $this->sendError($destFile,'fileCreate');
			}
		}

		/**
		 * Process folders
		 */
		foreach ($folders as $dir){
			$destFile=$destFolder.'/'.$dir;
			$sourceFile=$sourceFolder.'/'.$dir;

			// Skip excluded directories
			if(in_array($sourceFile,$this->excludedSourceFolders)) continue;

			// Create folder if destination doesn't exist
			if(!is_dir($destFile)){
				$this->sendAction($sourceFile,$destFile,'create');
				if($this->debug || @mkdir($destFile)) {
					$this->created++;
					// Recurse process created folder
					$this->updateDestination($sourceFile);
				}
				elseif (!$this->debug) $this->sendError($destFile,'folderCreate');
			}

			// Recurse process folder
			else $this->updateDestination($sourceFile);
		}
	}

	/**
	 * @desc Echo action on file / folder
	 *
	 * @param string $file: destination file or folder
	 * @param $string $action: 'skip', 'delete', 'update', 'create'
	 */
	function sendAction($sourceFile,$destFile,$action){
		if($this->outputFunction){
			$display=call_user_func($this->outputFunction, $sourceFile, $destFile, $action);
			if(!$display) return ;
		}
		if(isset($display) || $action!='skip') {
			echo $this->tag($action);
			echo $action.' '.$sourceFile.$this->lineBreak;
			echo $this->tag();
		}
		flush();
	}

	/**
	 * @desc Echo error message
	 *
	 * @param string $file: destination file or folder
	 * @param string $type: error type
	 */
	function sendError($file,$type){
		echo 'ERROR: '.$type.' > '.$file.$this->lineBreak;
		flush();
	}

	/**
	 * @desc Display a report on processed files
	 *
	 */
	function report(){
		echo '--------------------------------'.$this->lineBreak;
		echo 'Processed files: '.$this->processed.$this->lineBreak.$this->lineBreak;
		echo 'Created files: '.$this->created.$this->lineBreak;
		echo 'Updated files: '.$this->updated.$this->lineBreak;
		echo 'Removed files: '.$this->removed.$this->lineBreak;
		echo '--------------------------------'.$this->lineBreak;
	}
}